# -*- coding: utf-8 -*-
"""
Created on Mon Jul 19 13:30:56 2021

@author: 11426
"""

from  scipy.stats import chi2_contingency
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from pandas import DataFrame,Series
from sklearn.linear_model import LinearRegression
from sklearn.model_selection import train_test_split
plt.rcParams['font.sans-serif'] = ['SimHei']#声明黑体解决中文乱码,这里有好几种方式，篇幅有限，一种就够了
plt.rcParams['axes.unicode_minus'] = False

def chi2_Contingency(list1=[],list2=[],alpha=0.05):
    kf_data = np.array([list1, list2])
    kf = chi2_contingency(kf_data)
    print('检验两个类别变量是否独立：原假设：事件1和事件2是相互独立的事件')
    print('检验结果：chisq-statistic=%.4f, p-value=%.4f, df=%i expected_frep=%s'%kf)
    if kf[1] >= alpha:
        print('原假设成立，事件1和事件2相互独立')
    else:
        print('拒绝原假设，事件1和事件2不相互独立')
        
def Multiple_linear_regression(data,x,y,train_size = 0.6):
    print('数据描述：')
    print('head:',data.head(),'\nShape:',data.shape)
    print(data.describe())
    print('缺失值检验：')
    print(data[data.isnull()==True].count())
    print('数据箱型图：')
    data.boxplot()
    plt.xlabel('指标')
    plt.ylabel('指标数值')
    print('数据线性相关性：')
    f,ax = plt.subplots(figsize = (10,5))
    sns.heatmap(data.corr(),annot = True)
    print('建模分析：')
    X_train,X_test,Y_train,Y_test = train_test_split(x,y,train_size=train_size)
    model = LinearRegression()
    model.fit(X_train,Y_train)
    a  = model.intercept_#截距
    b = model.coef_#回归系数
    print("最佳拟合线:截距",a,",回归系数：",b)
    score = model.score(X_test,Y_test)
    print("R方检测:",score)
    print('对线性回归进行预测：')
    Y_pred = model.predict(X_test)
    plt.figure(figsize=(10,5))
    plt.plot(range(len(Y_pred)),Y_pred,'b',label="predict",linestyle='-.',alpha=0.5)
    plt.plot(range(len(Y_pred)),Y_test,'r',label='test',linestyle='-',alpha=0.5)
    plt.legend()
    plt.xlabel("测试次序")
    plt.ylabel("预测值")
    #显示图像
    plt.show()


# -*- coding: utf-8 -*-

from fbprophet import Prophet
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import datetime
from fbprophet.plot import add_changepoints_to_plot
from statsmodels.graphics.tsaplots import plot_acf, plot_pacf
from statsmodels.tsa.stattools import adfuller
from statsmodels.stats.diagnostic import acorr_ljungbox
from statsmodels.tsa.arima_model import ARIMA 
import warnings
warnings.filterwarnings(action = 'ignore')#忽略代码运行过程中的警告信息

def Forecast_Time_Serise(df,freq='1d',periods=30):
    df.columns = ['ds','y']
    df['ds'] = pd.to_datetime(df['ds'])
    m = Prophet()
    m.fit(df)
    future = m.make_future_dataframe(periods=periods, freq=freq)
    future.tail()
    forecast = m.predict(future)
    fig = m.plot(forecast)
    a = add_changepoints_to_plot(fig.gca(), m, forecast)
    x1 = forecast['ds']
    y1 = forecast['yhat']
    y2 = forecast['yhat_lower']
    y3 = forecast['yhat_upper']
    plt.plot(x1,y1)
    plt.plot(x1,y2)
    plt.plot(x1,y3)
    plt.show()
    m.plot_components(forecast)
    print('所有预测结果为：\n',forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']])
    print('最后五个预测结果为：\n',forecast[['ds', 'yhat', 'yhat_lower', 'yhat_upper']].tail())
    

def ARIMA_Prefare(data,n):   
    plt.rcParams['font.sans-serif'] = ['SimHei']    #定义使其正常显示中文字体黑体
    plt.rcParams['axes.unicode_minus'] = False      #用来正常显示表示负号
    print('数据概览：')
    plt.plot(data)
    plt.show()
    print('自相关性图：')
    #画出自相关性图
    plot_acf(data)
    plt.show()
    #平稳性检测
    print('原始序列的平稳性检验结果为：',adfuller(data.iloc[:,0:1]))
    print('返回值依次为：adf, pvalue p值， usedlag, nobs, critical values临界值 , icbest, regresults, resstore')
    print('若adf分别大于3中不同检验水平的3个临界值，单位检测统计量对应的p值显著大于 0.05 ，说明序列可以判定为非平稳序列')
    #对数据进行差分后得到 自相关图和 偏相关图
    D_data = data.diff().dropna()
    print('差分后的时序图：')
    fig = plt.figure(figsize=(6.5,4.5))
    plt.plot(D_data)   #画出差分后的时序图
    plt.show()
    print('差分后的自相关图：')
    plot_acf(D_data)    #画出自相关图
    plt.show()
    print('差分后的偏相关图：')
    plot_pacf(D_data)   #画出偏相关图
    plt.show()
    print(u'差分序列的ADF 检验结果为： ', adfuller(D_data.iloc[:,0:1]))   #平稳性检验
    print('一阶差分后的序列的时序图在均值附近比较稳的波动， 自相关性有很强的短期相关性， 单位根检验 p值小于 0.05 ，所以说一阶差分后的序列是平稳序列')
    print(u'差分序列的白噪声检验结果（第二项为p值）：',acorr_ljungbox(D_data, lags= 1))


def ARIMA_Model(data,pmax,qmax,n):
    bic_matrix = []
    for p in range(pmax +1):
        temp= []
        for q in range(qmax+1):
            try:
                temp.append(ARIMA(data.iloc[:,0:1], (p, 1, q)).fit().bic)
            except:
                temp.append(None)
            bic_matrix.append(temp)

    bic_matrix = pd.DataFrame(bic_matrix)   #将其转换成Dataframe 数据结构
    p,q = bic_matrix.stack().idxmin()   #先使用stack 展*， 然后使用 idxmin 找出最小值的位置
    print(u'BIC 最小的p值 和 q 值：%s,%s' %(p,q))  #  BIC 最小的p值 和 q 值：0,1
    #所以可以建立ARIMA 模型，ARIMA(0,1,1)
    model = ARIMA(data, (p,1,q)).fit()
    print('模型报告：')
    model.summary2()        #生成一份模型报告
    print(f'为未来{n}天进行预测，返回预测结果，标准误差和置信区间：')
    model.forecast(n)   #为未来5天进行预测， 返回预测结果， 标准误差， 和置信区间



# -*- coding: utf-8 -*-

import jieba
from wordcloud import WordCloud, STOPWORDS
import matplotlib.pyplot as plt
from snownlp import sentiment
from snownlp import SnowNLP
from nltk.tokenize import RegexpTokenizer
from stop_words import get_stop_words
from nltk.stem.porter import PorterStemmer
from gensim import corpora, models
import gensim


def draw_word_cloud(word):
    words = jieba.cut(word)
    wordstr = " ".join(words)
    sw = set(STOPWORDS)
    wc = WordCloud(
        font_path='C:/Windows/Fonts/simhei.ttf',  # 设置字体格式
        max_words=300,
        max_font_size=70,
        stopwords=sw,
        background_color='white',
        scale=20,
    ).generate(wordstr)
    # 显示词云图 
    plt.imshow(wc.recolor())
    plt.axis("off")
    plt.show()

def analysis_text_expression(text):
    s = SnowNLP(text)
    if s.sentiments <= 0.4: 
        commment = '表现为消极情绪倾向'
    elif s.sentiments > 0.4 and s.sentiments <= 0.6:
        commment = '表现为中立情绪倾向'
    else:
        commment = '表现为积极情绪倾向'
    print('该文本的情感倾向得分为：' ,s.sentiments,commment) 
    
def lda_model(text_url,num_topics=10,num_words=3):
    tokenizer = RegexpTokenizer(r'\w+')
    en_stop = get_stop_words('en')
    p_stemmer = PorterStemmer()
    with open(text_url, "r",encoding = 'utf-8') as f:
        text = f.read()
    doc_set = [text]
    texts = []

    for i in doc_set:
        raw = i.lower()
        tokens = tokenizer.tokenize(raw)
        stopped_tokens = [i for i in tokens if not i in en_stop]
        stemmed_tokens = [p_stemmer.stem(i) for i in stopped_tokens]
        texts.append(stemmed_tokens)

    dictionary = corpora.Dictionary(texts)
    corpus = [dictionary.doc2bow(text) for text in texts]
    lda = models.ldamodel.LdaModel(corpus=corpus, id2word=dictionary, num_topics=num_topics)
    for topic in lda.print_topics(num_topics=num_topics, num_words=num_words):
        print(topic)
    return lda

def keywords(text_url,num_keywords=10):
    with open(text_url, "r",encoding = 'utf-8') as f:
        text = f.read()
    text = SnowNLP(text)
    print(text.keywords(limit=num_keywords))
    
